@@ -22,7 +22,7 @@ from dictshield.fields import (StringField,
 from dictshield.fields import ObjectIdField
 
 import auth
-from brubeck import curtime
+from request_handling import curtime
 
 ###
 ### User Document
@@ -49,18 +49,21 @@ class User(Document):
         return u'%s' % (self.username)
 
     def set_password(self, raw_passwd):
-        """Generates bcrypt hash and salt for storing a user's password
+        """Generates bcrypt hash and salt for storing a user's password. With
+        bcrypt, the salt is kind of redundant, but this format stays friendly
+        to other algorithms.
         """
-        algorithm = auth.BCRYPT
-        (salt, digest) = auth.gen_hexdigest(algorithm, raw_passwd)
+        (algorithm, salt, digest) = auth.gen_hexdigest(raw_passwd)
         self.password = auth.build_passwd_line(algorithm, salt, digest)
 
-    def check_password(self, raw_password, passwd_line):
+    def check_password(self, raw_password):
         """Compares raw_password to password stored for user. Updates
         self.last_login on success.
         """
         algorithm, salt, hash = auth.split_passwd_line(self.password)
-        if hash == auth.gen_hexdigest(algorithm, salt, raw_password):
+        (_, _, user_hash) = auth.gen_hexdigest(raw_password,
+                                               algorithm=algorithm, salt=salt)
+        if hash == user_hash:
             self.last_login = curtime()
             return True
         else:
@@ -108,7 +111,7 @@ class UserProfile(Document):
     avatar_url = URLField(max_length=255)
 
     _private_fields = [
-        'user',
+        'owner',
     ]
 
     def __init__(self, *args, **kwargs):
