@@ -1,7 +1,8 @@
 from eventlet.green import zmq
 import time
 import json
-
+from uuid import uuid4
+import cgi
 
 ###
 ### Request handling code
@@ -23,16 +24,44 @@ class Request(object):
         self.conn_id = conn_id
         self.headers = headers
         self.body = body
-        
-        if self.headers['METHOD'] == 'JSON':
-            self.data = json.loads(body)
-        else:
-            self.data = {}
+
+        # populate arguments with QUERY string
+        self.arguments = {}
+        if 'QUERY' in self.headers:
+            query = self.headers['QUERY']
+            arguments = cgi.parse_qs(query)
+            for name, values in arguments.iteritems():
+                values = [v for v in values if v]
+                if values: self.arguments[name] = values
+
+        # handle data, multipart or not
+        if self.method in ("POST", "PUT") and self.content_type:
+            form_encoding = "application/x-www-form-urlencoded"
+            if self.content_type.startswith(form_encoding):
+                arguments = cgi.parse_qs(self.body)
+                for name, values in arguments.iteritems():
+                    values = [v for v in values if v]
+                    if values:
+                        self.arguments.setdefault(name, []).extend(values)
+            # Not ready for this, but soon
+#            elif content_type.startswith("multipart/form-data"):
+#                fields = content_type.split(";")
+#                for field in fields:
+#                    k, sep, v = field.strip().partition("=")
+#                    if k == "boundary" and v:
+#                        self._parse_mime_body(v, data)
+#                        break
+#                else:
+#                    logging.warning("Invalid multipart/form-data")
 
     @property
     def method(self):
         return self.headers.get('METHOD')
 
+    @property
+    def content_type(self):
+        return self.headers.get("content-type")
+
     @property
     def version(self):
         return self.headers.get('VERSION')
@@ -84,21 +113,24 @@ def http_response(body, code, status, headers):
 
 class Mongrel2Connection(object):
 
-    def __init__(self, sender_id, pull_addr, pub_addr):
-        """sender_id = uuid.uuid1() or anything unique
+    def __init__(self, pull_addr, pub_addr):
+        """sender_id = uuid.uuid4() or anything unique
         pull_addr = pull socket used for incoming messages
         pub_addr = publish socket used for outgoing messages
 
         The class encapsulates socket tupe by referring to it's pull socket
         as in_sock and it's publish socket as out_sock.
         """
-        self.sender_id = sender_id
+
+        # Each Brubeck instance uniquely identifies itself. Mongrel2 requires
+        # this for the request handler's pub socket as a subscriber id.
+        self.sender_id = uuid4().hex
 
         in_sock = CTX.socket(zmq.PULL)
         in_sock.connect(pull_addr)
 
         out_sock = CTX.socket(zmq.PUB)
-        out_sock.setsockopt(zmq.IDENTITY, sender_id)
+        out_sock.setsockopt(zmq.IDENTITY, self.sender_id)
         out_sock.connect(pub_addr)
 
         self.in_addr = pull_addr
@@ -120,11 +152,13 @@ class Mongrel2Connection(object):
         """
         header = "%s %d:%s," % (uuid, len(str(conn_id)), str(conn_id))
         #self.out_sock.send_pyobj(header + ' ' + msg)
-        self.out_sock.send(header + ' ' + msg)
+        #self.out_sock.send(header + ' ' + msg)
+        self.out_sock.send_unicode(header + ' ' + msg)        
 
     def reply(self, req, msg):
         """Does a reply based on the given Request object and message.
         """
+
         self.send(req.sender, req.conn_id, msg)
 
     def reply_bulk(self, uuid, idents, data):
